import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from sklearn.model_selection import train_test_split
from sklearn import tree
import sklearn.metrics as metrics
from itertools import combinations
from sklearn.ensemble import RandomForestClassifier
from sklearn.cluster import KMeans


data = pd.read_csv('Datas/Dtree/data_destree.csv', sep=',')


# Features
temp_ext = data['Exterieur_Temperature']
hum_ext = data['Exterieur_Humidite']
temp_ch1 = data['Ch1_Temperature']
hum_ch1 = data['Ch1_Humidite']
C02_ch1 = data['bbxco2_ch1']

Features = ['Exterieur_Temperature', 'Exterieur_Humidite', 'Ch1_Temperature', 'Ch1_Humidite', 'bbxco2_ch1']


ouverture_ch1 = data['Ch1_Ouverture']  # label

# ---- création de l'arbre de décision  ---
x_train, x_test, y_train, y_test = train_test_split(data[Features], data[['Ch1_Ouverture']],
                                                    test_size=0.3, random_state=0)
clf = tree.DecisionTreeClassifier(random_state=0, max_depth=8)
clf = clf.fit(x_train, y_train)
y_pred = clf.predict(x_test)
with open("Datas/Dtree/tree_ouverture.dot", 'w') as f:
    f = tree.export_graphviz(clf, out_file=f)

importance = clf.feature_importances_
print('Importance des features : \n')
for i in range(len(importance)):
    print(f'{Features[i]}, Importance: {round(importance[i], 4)}')

print('\n')
print('Precision: ', round(metrics.precision_score(y_test, y_pred, average="weighted"), 4))
print('F-Score: ', round(metrics.f1_score(y_test, y_pred, average="weighted"), 4))
print('Accuracy: ', round(metrics.accuracy_score(y_test, y_pred), 4))

print("\nMatrice des critères en fonction des valeurs des labels: ")
print(metrics.classification_report(y_test, y_pred, target_names=['fenetre fermée', 'fenetre ouverte 1fois', 'fenetre ouverte 2fois','fenetre ouverte 3fois', 'fenetre ouverte 4fois']))



# ---- précision en fonction de la profondeur maximale ---
accuracy = []
f_score = []
precision = []
accuracy_0 = []
accuracy_1 = []
accuracy_2 = []
accuracy_3 = []
accuracy_4 = []

max_depth = 30

for i in range(2, max_depth):
    clf = tree.DecisionTreeClassifier(random_state=0, max_depth=i)
    clf = clf.fit(x_train, y_train)
    y_pred3 = clf.predict(x_test)

    accur = round(metrics.accuracy_score(y_test, y_pred3), 4)
    f1 = round(metrics.f1_score(y_test, y_pred3, average="weighted"), 4)
    prec = round(metrics.precision_score(y_test, y_pred3, average="weighted"), 4)

    metrics.classification_report(y_test, y_pred,
                                  target_names=['fenetre fermée', 'fenetre ouverte 1fois', 'fenetre ouverte 2fois',
                                                'fenetre ouverte 3fois', 'fenetre ouverte 4fois'])

    accuracy.append(accur)
    precision.append(prec)
    f_score.append(f1)

plt.plot(range(2, max_depth), accuracy, label="accuracy", color="red")
# plt.plot(range(2, max_depth), f_score, label="f_score", color="blue")
# plt.plot(range(2, max_depth), precision, label="precision", color="green")
plt.xlabel("Profondeur max de l'arbre")
plt.legend()
plt.show()

#  ---------- Etude de différents scénarios avec différentes séries ----------
comb = []

for i in range(len(Features) + 1):
    comb_element = [list(k) for k in combinations(Features, i)]
    for k in range(len(comb_element)):
        comb.append(comb_element[k])

comb = comb[1:]  # on enlève le premier élement car il s'agit de la combinaison vide

accuracy_scenarios = []

for combination in comb:
    x_train4, x_test4, y_train4, y_test4 = train_test_split(data[combination], data[['Ch1_Ouverture']], test_size=0.3,
                                                            random_state=0)
    clf = tree.DecisionTreeClassifier(random_state=0, max_depth=8)
    clf = clf.fit(x_train4, y_train4)
    y_pred4 = clf.predict(x_test4)
    accuracy = round(metrics.accuracy_score(y_test4, y_pred4), 4)
    accuracy_scenarios.append(accuracy)
    print(comb.index(combination) + 1, combination, "--> accuracy: ", accuracy)

#plt.scatter(range(len(comb)), accuracy_scenarios)
plt.plot(range(len(comb)), accuracy_scenarios)
plt.xlabel("scenario (n°)")
plt.ylabel("global accuracy")
plt.title('accuracy pour tous les scenarios')
plt.show()




